'''
Rhine Python Library 
(c) Speare Inc. 2015
'''

try:
  import ujson as json
except ImportError:
  import json

import requests

from .exceptions import *

class RhineInstance:
  def __init__(self, apikey, ssl = False):
    self.apikey = apikey
    self.url = 'https://api.rhine.io' if ssl else 'http://api.rhine.io/'

  def __str__(self):
    return 'RhineInstance<apikey:{0}>'.format(self.apikey)

  def _run(self, req):
    result = requests.post(self.url, data = json.dumps({'request': {'method': req, 'key': self.apikey}})).json()
    if 'success' in result:
      return result['success']
    else:
      if result['failure'] == 'invalidjson': raise InvalidRequest()
      if result['failure'] == 'invalidauthentication': raise InvalidAuthentication()
      if result['failure'] == 'internalerror': raise InternalError()

  def run(self, req):
    name = list(req.keys())[0]
    return self._run(req)[name]

  def pipeline(self, reqs):
    return [x[0][x[1]] for x in zip(self._run({'pipelined': reqs})['multiple'], [list(r.keys())[0] for r in reqs])]    

def instantiate(key, ssl = False):
  return RhineInstance(key, ssl)



'''
Rhine Python Library
(c) Speare Inc. 2015
'''

extraction = lambda o: {'extraction': o}
closest = lambda o: {'closest': o}
equivalence = lambda x, y: {'equivalence': (x, y)}
subclass = lambda x, y: {'subclass': (x, y)}
distance = lambda x, y: {'distance': (x, y)}
clustering = lambda os: {'clustering': os}



'''
Rhine Python Library
(c) Speare Inc. 2015
'''

class article:
  fromurl = staticmethod(lambda url: {'article': {'fromurl': url}})

class image:
  fromurl = staticmethod(lambda url: {'image': {'fromurl': url}})

text = lambda t: {'text': t}

entity = lambda e: {'entity': e}

grouped = lambda *os: {'grouped': os}



'''
Rhine Python Library
(c) Speare Inc. 2015
'''

class RhineException(Exception):
  pass

class InvalidAuthentication(RhineException):
  pass

class InvalidRequest(RhineException):
  pass

class InternalError(RhineException):
  pass



from setuptools import setup

setup (
  name = 'rhine',
  version = '1.0.1',
  description = 'Rhine Python Client',
  long_description = 'For documentation, please see [http://www.rhine.io/docs].',
  url = 'https://github.com/Speare/rhine-python',
  license = 'New BSD',
  author = 'Speare Inc.',
  author_email = 'admin@rhine.io',
  packages = ['rhine'],
  package_dir = {'rhine': 'rhine'},
  include_package_data = True,
  install_requires = ['requests'],
  classifiers = [
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3'
  ]
)
